
/*				<<<<Last Modified: Thu Feb 08 15:07:16 1996>>>>
------------------------------------------------------------------------------

	=====
	CPCfs -- CPCEmu Filesystem Maintenance
        =====                   especially for CPCEmu

	Version 0.85                    (c) February '96 by Derik van Zuetphen
------------------------------------------------------------------------------
*/


#include "cpcfs.h"

/***********
  Variables
 ***********/

/****** CPC Filesystem ******/

char	full_imagename[FULL_IMAGENAME_LENGTH];	/* full pathname, ... */
char	*imagename;		/* ... only name portion, and ... */
int	imagefile;		/* ... handle of imagefile */

int	cur_user	= 0;
int	cur_format;
int	BLKNR_SIZE 	= 1;
int	BLKNR 		= 16;

uchar	*track = NULL;	/* 0x100 header + track data */
unsigned char filler;	/* copied from track header */

uchar	*block_buffer = NULL;	/* Buffer for 1024 or more bytes */

uchar	*blk_alloc = NULL;	/* Block-Allocation-Table, */
				/* chars point to fcb, FF = free block */
int	allocated_blks;
int	free_blks;
int	total_blks;	/* = allocated_blks + free_blks */
float	percentage;	/* of allocated blocks */

int	mode;		/* Mode for GET-commands */
bool	force;

struct d_header disk_header;


/****** User Interface *****/
int	nbof_args;
char	*arg[INPUTLEN];
char	prompt[INPUTLEN];
int	pagelen, lineno;
bool	Break_Wish;
jmp_buf break_entry;
bool	Interactive;
char	installpath[INPUTLEN];		/* path of argv[0] */
int	Verb = 9;

DirEntry *directory = (DirEntry*)NULL;

/* KT - adjusted stamp */
char stamp[] = "Compiled: " STAMP ", by Kevin Thacker.";
char copyright[] = "Original code (C) Dec. 1995 Derik van Ztphen";


/****** Disk Parameter Block ******/
#if 0
/* DPB templates for SYSTEM, DATA, IBM, VORTEX, and user defined */
#define DPB_store_size  5
DPB_type DPB_store[DPB_store_size] = {
/* SYSTEM */
	{	/*SYSTEMFORMAT,*/ /* ID */
		0x041, /* SEC1 */
		9,	/* SECS */
		40,	/* TRKS */
		1,	/* HDS */
		512,	/* BPS */

		36,	/* SPT */
		3,	/* BSH */
		7,	/* BLM */
		0,	/* EXM */
		168,	/* DSM */
		63,	/* DRM */
		0xC0,	/* AL0 */
		0x00,	/* AL1 */
		16,	/* CKS */
		2,	/* OFS */
		0,0,0
	},
/* DATA */
	{	/*DATAFORMAT,*/ /* ID */
		0x0c1, /* SEC1 */
		9,	/* SECS */
		40,	/* TRKS */
		1,	/* HDS */
		512,	/* BPS */

		36,	/* SPT */
		3,	/* BSH */
		7,	/* BLM */
		0,	/* EXM */
		177,	/* DSM */
		63,	/* DRM */
		0xC0,	/* AL0 */
		0x00,	/* AL1 */
		16,	/* CKS */
		0,	/* OFS */
		0,0,0
	},
/* IBM */
	{	/*IBMFORMAT,*/ /* ID */
		0x01, /* SEC1 */
		8,	/* SECS */
		40,	/* TRKS */
		1,	/* HDS */
		512,	/* BPS */

		36,	/* SPT */
		3,	/* BSH */
		7,	/* BLM */
		0,	/* EXM */
		153,	/* DSM */
		63,	/* DRM */
		0xC0,	/* AL0 */
		0x00,	/* AL1 */
		16,	/* CKS */
		1,	/* OFS */
		0,0,0
	},
/* VORTEX */
	{       /*VORTEXFORMAT,*/ /* ID */
		0x01,	/* SEC1 */
		9,	/* SECS */
		80,	/* TRKS */
		2,	/* HDS */
		512,	/* BPS */

		36,	/* SPT */
		5,	/* BSH */
		31,	/* BLM */
		3,	/* EXM */
		176,	/* DSM */
		127,	/* DRM */
		0x80,	/* AL0 */
		0x00,	/* AL1 */
		32,	/* CKS */
		2,	/* OFS */
		0,0,0
	}
/* user defined DPB is empty for now */
};
#endif

DPB_list_entry *dpb_list_entry = NULL;
DPB_type *dpb = NULL;	/* pointer to current DPB */
DPB_list dpb_list;

void	dpb_list_add_entry(DPB_list *list, DPB_list_entry *entry)
{
	DPB_list_entry *cur_entry;

	/* if list empty add as first item */
	if (list->first==NULL)
	{
		list->first = entry;
	}
	else
	{
		/* add to end of list */
		cur_entry = list->first;

		while (cur_entry->next!=NULL)
		{
			cur_entry = cur_entry->next;
		}

		cur_entry->next = entry;
	}
}


/* remove entry from list */
void	dpb_list_remove_entry(DPB_list *list,DPB_list_entry *entry)
{
	DPB_list_entry *cur_entry;

	/* entry at start of list? */
	if (list->first==entry)
	{
		list->first = entry->next;
		return;
	}

	/* find entry which points to this one and update it's next pointer
	to point to next of entry we are removing */
	cur_entry = list->first;

	while (cur_entry!=NULL)
	{
		if (cur_entry->next == entry)
		{
			cur_entry->next = entry->next;
			return;
		}
	}
}

/* delete an entry from the list */
void	dpb_list_entry_delete(DPB_list *list,DPB_list_entry *entry)
{
	/* remove it */
	dpb_list_remove_entry(list, entry);

	/* delete entry data */
	if (entry->description!=NULL)
	{
		free(entry->description);
	}

	if (entry->ident!=NULL)
	{
		free(entry->ident);
	}

	/* delete entry */
	free(entry);
}

/* delete all items in list */
void	dpb_list_delete(DPB_list *list)
{
	DPB_list_entry *entry;

	entry = list->first;

	while (entry!=NULL)
	{
		/* get next entry */
		DPB_list_entry *next = entry->next;

		/* delete the entry */
		dpb_list_entry_delete(list,entry);

		entry = next;
	}
}


void	dpb_list_add_item(DPB_list *list,char *description, char *ident, DPB_type *dpb)
{
	DPB_list_entry *entry = (DPB_list_entry *)malloc(sizeof(DPB_list_entry));

	if (entry!=NULL)
	{
		if (description==NULL)
		{
			entry->description = NULL;
		}
		else
		{
			entry->description = (char *)malloc(strlen(description)+1);

			if (entry->description!=NULL)
			{
				strcpy(entry->description, description);
			}
		}

		if (ident==NULL)
		{
			entry->ident = NULL;
		}
		else
		{
			entry->ident = (char *)malloc(strlen(ident)+1);

			if (entry->ident!=NULL)
			{
				strcpy(entry->ident, ident);
			}
		}


		/* copy dpb */
		memcpy(&entry->dpb, dpb, sizeof(DPB_type));

		entry->next = NULL;
	}

	dpb_list_add_entry(list, entry);
}


/* KT - moved to here because it was causing problems with MSDev */
int	ui_main (int,char**);

void parse_def_file(char *);

void main (int argc, char **argv) {
	/* initialise list */
	dpb_list.first = NULL;

	parse_def_file("cpmdisks.def");
										  
	ui_main (argc,argv);
	
	dpb_list_delete(&dpb_list);

	exit(0);
}
